// 获取画布容器和画布元素
const container = document.getElementById('3dcontainer');
const canvas = document.getElementById('myCanvas');
// 创建场景
const scene = new THREE.Scene();
// 创建透视相机
const camera = new THREE.PerspectiveCamera(75, window.innerWidth / window.innerHeight, 0.1, 1000);
camera.position.z = 5; // 设置相机位置
// 创建渲染器并添加到DOM中
const renderer = new THREE.WebGLRenderer({ antialias: true });
renderer.setSize(window.innerWidth, window.innerHeight);
container.appendChild(renderer.domElement);
// 创建环境光
const ambientLight = new THREE.AmbientLight(0xcccccc, 0.4);
scene.add(ambientLight);
// 创建点光源
const pointLight = new THREE.PointLight(0xffffff, 0.8);
pointLight.position.set(10, 10, 10); // 设置光源位置
scene.add(pointLight);
// 使用GLTFLoader加载模型
const loader = new THREE.GLTFLoader();
loader.load(
    'path_to_your_model.gltf', // 模型文件路径
    function (gltf) {
        scene.add(gltf.scene); // 将模型添加到场景中
    },
    undefined, // 进度回调函数（可选）
    function (error) {
        console.error(error); // 错误处理（可选）
    }
);
function animate() {
    requestAnimationFrame(animate); // 请求下一帧动画
    // 更新场景和相机
    renderer.render(scene, camera);
}
animate(); // 启动动画循环
window.addEventListener('resize', onWindowResize, false);
function onWindowResize() {
    camera.aspect = window.innerWidth / window.innerHeight;
    camera.updateProjectionMatrix();
    renderer.setSize(window.innerWidth, window.innerHeight);
}
// 假设你已经按照之前的步骤设置了场景、相机和渲染器

// 加载浣熊的纹理图片
const textureLoader = new THREE.TextureLoader();
const raccoonTexture = textureLoader.load('path_to_raccoon_texture.png');

// 创建浣熊的身体几何形状，这里使用一个圆柱体作为示例
const bodyGeometry = new THREE.CylinderGeometry(1, 1, 2, 32);
const headGeometry = new THREE.SphereGeometry(0.75, 32, 32);
const earGeometry = new THREE.CylinderGeometry(0.25, 0.5, 0.5, 32);

// 创建浣熊的耳朵
const earMaterial = new THREE.MeshPhongMaterial({ color: 0x5B3925 });
const leftEar = new THREE.Mesh(earGeometry, earMaterial);
leftEar.position.set(1.15, 1.25, 0.5);
const rightEar = leftEar.clone();
rightEar.position.set(-1.15, 1.25, 0.5);

// 创建浣熊的头部
const headMaterial = new THREE.MeshPhongMaterial({ color: 0x8B4513 });
const head = new THREE.Mesh(headGeometry, headMaterial);
head.position.set(0, 2, 0);

// 创建浣熊的身体
const bodyMaterial = new THREE.MeshPhongMaterial({ color: 0x8B4513 });
const body = new THREE.Mesh(bodyGeometry, bodyMaterial);
body.position.set(0, 0, 0);

// 创建浣熊的纹理材质
const raccoonMaterial = new THREE.MeshPhongMaterial({ map: raccoonTexture });

// 将纹理材质应用到浣熊的头部和身体
head.material = raccoonMaterial;
body.material = raccoonMaterial;

// 将浣熊的耳朵、头部和身体组合成一个组
const raccoonGroup = new THREE.Group();
raccoonGroup.add(leftEar);
raccoonGroup.add(rightEar);
raccoonGroup.add(head);
raccoonGroup.add(body);

// 将浣熊模型添加到场景中
scene.add(raccoonGroup);

// 更新渲染循环以包括浣熊模型的旋转
function animate() {
    requestAnimationFrame(animate);

    // 旋转浣熊模型，使其更生动
    raccoonGroup.rotation.y += 0.01;

    renderer.render(scene, camera);
}

animate();