// 获取窗口元素和控制按钮
const window1 = document.getElementById('window-1');
const startButton = document.getElementById('start-button');
const minimizeButton = document.querySelector('.minimize');
const maximizeButton = document.querySelector('.maximize');
const closeButton = document.querySelector('.close');

// 函数：切换开始菜单的显示与隐藏
function toggleStartMenu() {
    const startMenu = document.getElementById('start-menu');
    startMenu.classList.toggle('active');
}

// 函数：最小化窗口
function minimizeWindow() {
    window1.classList.add('minimized');
    window1.style.display = 'none'; // 隐藏窗口
    // 可以在这里更新任务栏缩略图的状态
}

// 函数：最大化窗口
function maximizeWindow() {
    window1.classList.add('maximized');
    window1.style.width = `${window.innerWidth - 10}px`;
    window1.style.height = `${window.innerHeight - 32}px`;
    window1.style.top = '0';
    window1.style.left = '0';
}

// 函数：恢复窗口到原始大小
function restoreWindow() {
    window1.classList.remove('minimized', 'maximized');
    window1.style.width = '300px';
    window1.style.height = '200px';
    window1.style.top = ''; // 清除之前设置的top值
    window1.style.left = ''; // 清除之前设置的left值
    window1.style.display = 'block'; // 显示窗口
}

// 为开始按钮添加点击事件监听器
startButton.addEventListener('click', toggleStartMenu);

// 为最小化按钮添加点击事件监听器
minimizeButton.addEventListener('click', minimizeWindow);

// 为最大化按钮添加点击事件监听器
maximizeButton.addEventListener('click', maximizeWindow);

// 为关闭按钮添加点击事件监听器
closeButton.addEventListener('click', () => {
    window1.style.display = 'none'; // 隐藏窗口
    // 可以在这里更新任务栏缩略图的状态
});

// 窗口拖动逻辑
let isDragging = false;
let dragStartX, dragStartY, dragOffsetX, dragOffsetY;

function onMouseDown(e) {
    e.preventDefault();
    isDragging = true;
    dragStartX = e.clientX;
    dragStartY = e.clientY;
    const windowRect = window1.getBoundingClientRect();
    dragOffsetX = dragStartX - windowRect.left;
    dragOffsetY = dragStartY - windowRect.top;
}

function onMouseMove(e) {
    if (!isDragging) return;
    e.preventDefault();
    const newX = e.clientX - dragOffsetX;
    const newY = e.clientY - dragOffsetY;
    window1.style.top = `${newY}px`;
    window1.style.left = `${newX}px`;
}

function onMouseUp() {
    isDragging = false;
}

window1.addEventListener('mousedown', onMouseDown);
document.addEventListener('mousemove', onMouseMove);
document.addEventListener('mouseup', onMouseUp);